/*
This file is part of the Notesnook project (https://notesnook.com/)

Copyright (C) 2023 Streetwriters (Private) Limited

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { initialize } from "@notesnook/sodium";
import Decryption from "./src/decryption";
import Encryption from "./src/encryption";
import KeyUtils from "./src/keyutils";
import Password from "./src/password";
export class NNCrypto {
    constructor() {
        this.isReady = false;
        // async encryptStream(
        //   key: SerializedKey,
        //   stream: IStreamable,
        //   _streamId?: string
        // ): Promise<string> {
        //   await this.init();
        //   return await this.createEncryptionStream(key, stream);
        // }
        // async decryptStream(
        //   key: SerializedKey,
        //   iv: string,
        //   stream: IStreamable,
        //   _streamId?: string
        // ): Promise<void> {
        //   await this.init();
        //   await this.createDecryptionStream(iv, key, stream);
        // }
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isReady)
                return;
            yield initialize();
            this.isReady = true;
        });
    }
    encrypt(key, plaintext, outputFormat = "uint8array") {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.init();
            return Encryption.encrypt(key, plaintext, outputFormat);
        });
    }
    decrypt(key, cipherData, outputFormat = "text") {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.init();
            return Decryption.decrypt(key, cipherData, outputFormat);
        });
    }
    hash(password, salt) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.init();
            return Password.hash(password, salt);
        });
    }
    deriveKey(password, salt) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.init();
            return KeyUtils.deriveKey(password, salt);
        });
    }
    exportKey(password, salt) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.init();
            return KeyUtils.exportKey(password, salt);
        });
    }
    createEncryptionStream(key) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.init();
            return Encryption.createStream(key);
            // // eslint-disable-next-line no-constant-condition
            // while (true) {
            //   const chunk = await stream.read();
            //   if (!chunk) break;
            //   const { data, final } = chunk;
            //   if (!data) break;
            //   const encryptedChunk: Chunk = {
            //     data: encryptionStream.write(data, final),
            //     final
            //   };
            //   await stream.write(encryptedChunk);
            //   if (final) break;
            // }
            // return encryptionStream.header;
        });
    }
    createDecryptionStream(key, iv) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.init();
            return Decryption.createStream(iv, key);
            // eslint-disable-next-line no-constant-condition
            // while (true) {
            //   const chunk = await stream.read();
            //   if (!chunk) break;
            //   const { data, final } = chunk;
            //   if (!data) break;
            //   const decryptedChunk: Chunk = {
            //     data: decryptionStream.read(data),
            //     final
            //   };
            //   await stream.write(decryptedChunk);
            //   if (final) break;
            // }
        });
    }
}
export * from "./src/types";
export * from "./src/interfaces";
