import { Cipher, EncryptionKey, OutputFormat, Plaintext, SerializedKey, Chunk } from "./types";
export interface IStreamable {
    read(): Promise<Chunk | undefined>;
    write(chunk: Chunk | undefined): Promise<void>;
}
export interface INNCrypto {
    encrypt(key: SerializedKey, plaintext: Plaintext, outputFormat?: OutputFormat): Promise<Cipher>;
    decrypt(key: SerializedKey, cipherData: Cipher, outputFormat?: OutputFormat): Promise<Plaintext>;
    hash(password: string, salt: string): Promise<string>;
    deriveKey(password: string, salt?: string): Promise<EncryptionKey>;
    exportKey(password: string, salt?: string): Promise<SerializedKey>;
    createEncryptionStream(key: SerializedKey): Promise<{
        iv: string;
        stream: TransformStream<Chunk, Uint8Array>;
    }>;
    createDecryptionStream(key: SerializedKey, iv: string): Promise<TransformStream<Uint8Array, Uint8Array>>;
}
